//---------------------------------------------------------------------------
#include <fmx.h>
#pragma hdrstop

#include "Main.h"
#include "FMX.Helpers.Android.hpp"
#include "Androidapi.Helpers.hpp"
#include "Winsoft.Android.Usb.hpp"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.fmx"
TFormMain *FormMain;
//---------------------------------------------------------------------------
__fastcall TFormMain::TFormMain(TComponent* Owner)
  : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::FormCreate(TObject *Sender)
{
  Device = new TSerialDevice();
  RefreshDevices();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::RefreshDevices()
{
  ComboBoxDevices->Clear();
  Device->EnumDevices();
  for (int I = 0; I < Device->DeviceCount; ++I)
  {
    TSerialDeviceInfo *DeviceInfo = Device->DeviceInfo[I];
    try
    {
      ComboBoxDevices->Items->Add(DeviceInfo->Description);
    }
    __finally
    {
      delete DeviceInfo;
    }
  }

  if (ComboBoxDevices->ItemIndex == -1)
  {
    Close();
    if (ComboBoxDevices->Items->Count > 0)
      ComboBoxDevices->ItemIndex = 0;
  }

  ComboBoxDevicesChange(this);
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ComboBoxDevicesChange(TObject *Sender)
{
  ButtonOpenClose->Enabled = ComboBoxDevices->ItemIndex != -1;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Open()
{
  Memo->Lines->Clear();
  Text = Memo->Text;
  Memo->SetFocus();
  Memo->GoToTextEnd();
  Memo->SelStart = Text.Length();
  Memo->SelLength = 0;

  // show library version
  AppendText("FTDI library version: " + IntToHex(TJD2xxManager::JavaClass->getLibraryVersion(), 8) + sLineBreak);

  // show device info
  TSerialDeviceInfo *DeviceInfo = Device->DeviceInfo[ComboBoxDevices->ItemIndex];
  try
  {
    AppendText("Device type: " + DeviceInfo->DeviceType + sLineBreak);
    AppendText("Description: " + DeviceInfo->Description + sLineBreak);
    AppendText("Serial number: " + DeviceInfo->SerialNumber + sLineBreak);
  }
  __finally
  {
    delete DeviceInfo;
  }

  // set communication parameters
  Device->DataBits = TDataBits::_8;
  Device->StopBits = TStopBits::_1;
  Device->Parity = TParity::None;
  Device->BaudRate = 9600;

  // open device
  Device->Open(ComboBoxDevices->ItemIndex);
  ComboBoxDevices->Enabled = false;
  ButtonOpenClose->Text = "Close";
  Memo->Enabled = true;
  Timer->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::Close()
{
  Timer->Enabled = false;
  Device->Close();
  ComboBoxDevices->Enabled = true;
  ButtonOpenClose->Text = "Open";
  Memo->Enabled = false;
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::ButtonOpenCloseClick(TObject *Sender)
{
  if (!Device->Opened)
    Open();
  else
    Close();
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::MemoChangeTracking(TObject *Sender)
{
  // Memo OnKeyDown doesn't work, so we need to track changes manually
  if (Device->Opened)
  {
    String MemoText = Memo->Text;
    if (MemoText != Text)
    {
      if (MemoText.Length() > Text.Length()) // character was added
      {
        Char KeyChar = '\0';
        bool Found = False;
        for (int I = 1; I <= Text.Length(); ++I)
          if (MemoText[I] != Text[I])
          {
            KeyChar = MemoText[I]; // first different character found
            Found = true;
            break;
          }

        if (!Found)
          KeyChar = MemoText[Text.Length() + 1];

        Device->Write(KeyChar);
      }
      Text = MemoText;
    }
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::TimerTimer(TObject *Sender)
{
  DynamicArray<Byte> Buffer;
  Buffer.Length = Device->InputCount();
  int Size = Device->Read(Buffer);
  if (Size >= 1)
  {
    Buffer.Length = Size;
    AppendText(TEncoding::ANSI->GetString(Buffer));
  }
}
//---------------------------------------------------------------------------
void __fastcall TFormMain::AppendText(String NewText)
{
  if (NewText.Length() == 0)
    return;

  Text = Memo->Text + NewText;
  Memo->Text = Text;
  Memo->GoToTextEnd();
  Memo->SelStart = Text.Length();
  Memo->SelLength = 0;
}
//---------------------------------------------------------------------------